<?php
require_once('data.inc.php');
require_once('export.inc.php');

/**
 * Convert one changelog entry object to a PHP array
 *
 * @param 	str $obj_type
 * @param 	str $obj (serialized)
 * @return	PHP array
 */
function convert_changelog_object($obj_type=false, $obj=false)
{
	if (empty($obj_type) || empty($obj)) {
		return false;
	}

	return @unserialize($obj);
}

/**
 * Check if an object has been changed by someone else since $timestamp.
 *
 * @return bool true if someone has changed it, false otherwise.
 */
function has_others_changed($obj_type, $obj_id)
{
	global $debug;
	if (!$obj_id || $obj_id == 'new')
		return false;

	if (!isset($_SESSION['nacoma_last_change'][$obj_type.'/'.$obj_id])) {
		if ($debug)
			print "AAARGH! How did you post to this page without having previously viewed it?";
		return false;
	}
	$timestamp = $_SESSION['nacoma_last_change'][$obj_type.'/'.$obj_id];
	$query = 'SELECT 1 AS a FROM changelog ' .
		'WHERE object_type = :type AND object_id = :oid AND username != :uname ' .
		'AND time > FROM_UNIXTIME(:timestamp) LIMIT 1';
	$res = nacoma_db::prepare_statement($query);
	$res->execute(array('type' => $obj_type, 'oid' => $obj_id, 'uname' => get_user(), 'timestamp' => $timestamp));
	$res = $res->fetchAll(PDO::FETCH_NUM);
	return !empty($res);
}

function store_last_viewed($obj_type, $obj_id)
{
	$res = nacoma_db::exec_query('SELECT UNIX_TIMESTAMP() AS time');

	$row = $res->fetch(PDO::FETCH_ASSOC);
	$time = $row['time'];
	unset($res);

	if (!isset($_SESSION['nacoma_last_change']))
		$_SESSION['nacoma_last_change'] = array();
	$_SESSION['nacoma_last_change'][$obj_type.'/'.$obj_id] = $time;
}

/**
 * Return query string that excludes all non-exported objects by any users
 * other than the current.
 *
 * Should be used like "AND id ".get_exclude_pending_string($obj_type) to
 * handle errors correctly.
 *
 * This must be used everywhere where object lists are being retrieved.
 */
function get_exclude_pending_string($obj_type) {
	return 'NOT IN (SELECT object_id FROM changelog WHERE oldstate IS NULL AND object_type = '.nacoma_db::quote_string($obj_type).' AND username != '.nacoma_db::quote_string(get_user()).')';
}

/**
 * Return TRUE if an object is pending, and FALSE if it isn't.
 * Unlike get_exclude_pending_string, this function doesn't care if the user
 * who created the pending object is the one currently logged in.
 */
function is_object_pending($obj_type, $obj_id) {
	$sql = 'SELECT 1 AS a FROM changelog WHERE oldstate IS NULL AND object_type = '.nacoma_db::quote_string($obj_type).' AND object_id = '.nacoma_db::quote_string($obj_id);
	$result = nacoma_db::exec_query($sql);
	$ret = false;
	if ($result->fetch(PDO::FETCH_NUM))
		$ret = true;
	unset($result);
	return $ret;
}


/**
 * Reverts a user's changes.
 *
 * There are a couple of different cases that needs to be considered
 *
 * - The simple case: the current user's changes are to objects no other users
 *   have pending changes on. In that case we could simply revert the changes,
 *   delete the changelog entries, and save.
 * - There's a half-annoying case, when the user and other users have edited
 *   the same object, but different fields. Unless the object name has been
 *   changed, the solution to the simple case applies.
 * - The annoyingly annoying case: the current user and other users have been
 *   editing the same field in the same object. In this case, we must do a
 *   complete rollback of the affected objects, followed by careful
 *   reapplication of other users' changes and regeneration of their changelog
 *   entries. This becomes extra annoying when this causes changelog entries to
 *   become empty.
 *
 * @return boolean false if no lock could be obtained
 */
function revert_users_changes($object_id = false) {
	if(!nacoma_objlock::extend_lock('all', 0, 5*60)) {
		return false;
	}
	$overlay = array();
	$deleted_overlay = array();
	$user = get_user();
	$quoteduser = nacoma_db::quote_string($user);
	$object_id = $object_id ? (int)$object_id : false;
	$query_affected = 'SELECT * FROM changelog WHERE (object_type, object_id) IN ' .
		'(SELECT object_type, object_id FROM changelog ' .
		'WHERE username = '.$quoteduser.') ' .
		($object_id ? 'AND object_id = ' . $object_id . ' ' : '') .
		'ORDER BY id DESC';
	$query_to_reapply = 'SELECT * FROM changelog WHERE (object_type, object_id) IN ' .
		'(SELECT object_type, object_id FROM changelog ' .
		'WHERE username = '.$quoteduser.') ' .
		($object_id ? 'AND object_id = ' . $object_id. ' '  : '') .
		'AND username != '.$quoteduser.' ' .
		'ORDER BY id ASC';
	$query_to_delete = 'DELETE FROM changelog WHERE username = '.$quoteduser.($object_id ? ' AND object_id = ' . $object_id : '');
	$stmt_update_changelog = nacoma_db::prepare_statement('UPDATE changelog SET oldname=:oldname, newname=:newname, oldstate=:oldstate, newstate=:newstate WHERE id=:id');
	$stmt_delete_changelog_entry = nacoma_db::prepare_statement('DELETE FROM changelog WHERE id=:id');

	# first, iterate over all changes backwards, to get to the oldest state for
	# all objects.
	$affected = nacoma_db::exec_query($query_affected);
	foreach($affected->fetchAll(PDO::FETCH_ASSOC) as $ary) {
		$class = $ary['object_type'].'_nacoma_Model';
		$fromobj = convert_changelog_object($ary['object_type'], $ary['newstate']);
		$toobj = convert_changelog_object($ary['object_type'], $ary['oldstate']);
		if (isset($overlay[$ary['object_type']][$ary['object_id']]) && $overlay[$ary['object_type']][$ary['object_id']] !== false)
			$obj = $overlay[$ary['object_type']][$ary['object_id']];
		else
			$obj = new $class($ary['object_id']);
		$newobj = convert_object_state($obj, $fromobj, $toobj);
		if ($newobj !== false) {
			if (!isset($overlay[$newobj->type]))
				$overlay[$newobj->type] = array();
			$overlay[$newobj->type][$newobj->id] = $newobj;
		} else {
			if (!isset($deleted_overlay[$ary['object_type']]))
				$deleted_overlay[$ary['object_type']] = array();
			$deleted_overlay[$ary['object_type']][$ary['object_id']] = $obj;
			$overlay[$ary['object_type']][$ary['object_id']] = false;
		}
	}

	# then, iterate over all changes forwards, excluding the current user's
	# changes, and reapply them to get the new state.
	$reapply = nacoma_db::exec_query($query_to_reapply);
	foreach ($reapply->fetchAll(PDO::FETCH_ASSOC) as $ary) {
		$class = $ary['object_type'].'_nacoma_Model';
		$fromobj = convert_changelog_object($ary['object_type'], $ary['oldstate']);
		$toobj = convert_changelog_object($ary['object_type'], $ary['newstate']);
		if ($overlay[$ary['object_type']][$ary['object_id']]) {
			$obj = $overlay[$ary['object_type']][$ary['object_id']];
			$oldname = $obj->get_object_name();
			$oldstate = $obj->obj;
		}
		else if ($deleted_overlay[$ary['object_type']][$ary['object_id']]) {
			$obj = $deleted_overlay[$ary['object_type']][$ary['object_id']];
			$oldname = '';
			$oldstate = false;
		}
		else {
			// object ID was reused from an object the user deleted after last
			// export
			$class = $ary['object_type'].'_nacoma_Model';
			$obj = new $class();
			$obj->id = $ary['object_id'];
			$oldname = '';
			$oldstate = false;
		}
		$obj = convert_object_state($obj, $fromobj, $toobj);
		if ($obj !== false) {
			if (!isset($overlay[$obj->type]))
				$overlay[$obj->type] = array();
			$overlay[$obj->type][$obj->id] = $obj;

			$newname = $obj->get_object_name();
			$newstate = $obj->obj;
			$obj->filter_objs_for_changelog($oldstate, $newstate);
		} else {
			unset($overlay[$ary['object_type']][$ary['object_id']]);
			$newname = $newstate = null;
		}

		if (!empty($oldstate) && !empty($newstate))
			$stmt_update_changelog->execute(array('oldname' => $oldname, 'newname' => $newname, 'oldstate' => serialize($oldstate), 'newstate' => serialize($newstate), 'id' => $ary['id']));
		else
			$stmt_delete_changelog_entry->execute(array('id' => $ary['id']));
	}

	// save final state of affected objects
	// (this might include unmodified objects, which is redundant but harmless)
	foreach($overlay as $type => $objs) {
		foreach ($objs as $id => $obj) {
			if ($obj == false) {
				$class = $type.'_nacoma_Model';
				$tmpobj = new $class($id);
				$tmpobj->banish_object(false);
			} else {
				$obj->save_object(false);
			}
		}
	}

	# finally delete the current user's changes
	nacoma_db::exec_query($query_to_delete);

	nacoma_objlock::free_lock('all', 0);
	return true;
}

function convert_object_state($obj, $oldstate, $newstate) {
	// handle added or removed custom vars
	$props = $obj->get_properties();
	if ($newstate) {
		foreach ($newstate as $key => $ary) {
			if (!isset($props['var_index'][$key]) && !empty($ary)) {
				$obj->custom_vars[$key] = $ary;
				unset($newstate[$key]);
				unset($oldstate[$key]);
			}
		}
	}
	if ($oldstate) {
		foreach ($oldstate as $key => $ary) {
			if (!isset($props['var_index'][$key])) {
				unset($obj->custom_vars[$key]);
				unset($oldstate[$key]);
			}
		}
	}
	// if this was actually a custom var entry, return early
	if (!$newstate && !$oldstate) {
		return $obj;
	}

	if ($newstate) {
		if ($obj->obj) {
			// handle added or removed relations
			foreach ($newstate as $key => $ary) {
				if (!is_array($ary))
					continue;
				if (!isset($obj->obj[$key]))
					continue;

				if (is_array($obj->obj[$key]) && is_array($oldstate[$key]))
					foreach ($oldstate[$key] as $idx => $val)
						unset($obj->obj[$key][array_search($val, $obj->obj[$key])]);

				foreach ($newstate[$key] as $idx => $val) {
					if (!in_array($val, $obj->obj[$key]))
						$obj->obj[$key][] = $val;
				}
				unset($newstate[$key]);
			}

			$obj->obj = array_merge($obj->obj, $newstate);
			foreach ($obj->obj as $k => $v) {
				if (!$v && $v !== '0' && $v !== 0)
					unset($obj->obj[$k]);
			}
		}
		else {
			$obj->obj = $newstate;
		}
		return $obj;
	}
	else if ($obj->type && $obj->id) {
		return false;
	} else {
		print "Wow, this is bad. I should delete an object, but I have no idea" .
		      " which object I should delete.";
		print "<pre>"; print_r($obj); print "</pre>";
		return false;
	}
}
