<?php

/**
 * Graph collections lets you create one page with a collection of different
 * graphs.
 *
 * This is different from Combined graphs, which creates one graph from
 * multiple data sources.
 *
 * You decide which graphs that should be included in a collection, either through
 * specifying a list of Services, or through enabling the Use of regex and
 * writing regular expressions for Host name and Service description.
 * The official documentation at
 * http://docs.pnp4nagios.org/pnp-0.6/pages includes more information about
 * regular expressions and collections in general.
 *
 * When you have configured your Graph collections, they will immediately be
 * visible in the Graphs UI. This means that you do not need to apply your
 * changes when dealing with Graph collections, they are generated as soon as
 * you save them.
 */
class graph_collection_nacoma_Model extends pnp_nacoma_Model {
	// @todo wow this should so not be hardcoded
	private static $page_dir = "/opt/monitor/etc/pnp/pages/";

	// stuff needed for inheritance compliance
	public $properties = array(
		'var_index' => array(
			'use_regex' => 'pnp_page_use_regex',
			'page_name' => 'string',
			'host_name' => 'string', // should be host_regex
			'service_desc' => 'string', // should be service_regex
			'services' => 'mselect' // should be services
		),
		'required' => array(
			'use_regex',
			'page_name'
		),
		'label' => array(
			'use_regex' => 'Use regex',
			'page_name' => 'Page name',
			'host_name' => 'Host name',
			'service_desc' => 'Service description',
			'hosts' => 'Hosts',
			'services' => 'Services'
		),
		'special' => array(
			'services' => array(), // to be filled with objects by get_properties
		),
		'topic' => 'page_name'
	);
	const TYPE = 'graph_collection';

	/**
	 * Return the mayi resource for the object
	 * @see object_nacoma_Model::mayi_resource()
	 */
	public function mayi_resource() {
		return "monitor.monitoring.graph_collections";
	}

	function __construct($id = false) {
		$this->type = static::TYPE;
		$this->get_properties();
		if (!$id)
			return;

		if (!$this->get_object_by_name($id))
			$this->id = false;
	}

	function get_object_by_name($name)
	{
		$name = basename($name, '.cfg');
		$this->id = $name;
		return $this->get_object();
	}

	private static function get_filename($page_name)
	{
		return self::$page_dir . static::objname_to_pnp($page_name) . '.cfg';
	}

	function get_properties()
	{
		$out = $this->get_available_graphs();
		$this->properties['special']['services'] = array_merge($out['hosts'], $out['services']);
		asort($this->properties['special']['services']);
		return $this->properties;
	}

	static public function object_exists($id)
	{
		return file_exists(self::get_filename($id));
	}

	/**
	 * Native pnp pages config into options array
	 *
	 * Nearly almost copy pasted from pnp's Data_nacoma_Model::parse_page_cfg()
	 *
	 * @param $data array lines of config file
	 * @return array
	 */
	private static function _config_to_options($data)
	{
		$inside=0;
		$allowed_tags = array("page", "graph");
		$options = array();
		$l = 0;
		foreach($data as $line){
			if(preg_match('/(^#|^;)/',$line)) {
				continue;
			}

			preg_match('/define\s+(\w+)\W+{/' ,$line, $tag);
			if(isset($tag[1]) && in_array($tag[1],$allowed_tags)){
				$inside = 1;
				$t = $tag[1];
				$l++;
				continue;
			}
			if(preg_match('/\s?(\w+)\s+([^#]+)(#.*)?$/',$line, $key) && $inside == 1){
				$k=strtolower($key[1]);
				$v=$key[2];
				if($t=='page'){
					$options[$k] = trim($v);
				}elseif($t=='graph'){
					$options[$l][$k] = trim($v);
				}
			} elseif(preg_match('/}/',$line)){
				$inside=0;
				$t = "";
				continue;
			}
		}
		return $options;
	}

	function get_object() {
		$filename = self::get_filename($this->id);
		if (!is_writable($filename)) {
			return false;
		}
		$file = file($filename);
		# If nacoma didn't generate it, then I can't touch this
		if (substr($file[0], 0, 7) !== '#NACOMA')
			return false;

		$options = $this->_config_to_options($file);
		$this->obj['use_regex'] =  isset($options['use_regex']) && $options['use_regex'];
		if ($options['page_name'])
			$this->obj['page_name'] =  $options['page_name'];

		if($this->obj['use_regex']) {
			// we're reading a saved config and we want the graph part
			// i.e. the only numeric index in that array
			foreach($options as $key => $values) {
				if(!is_numeric($key)) {
					continue;
				}
				$this->obj['host_name'] = $values['host_name'];
				$this->obj['service_desc'] = $values['service_desc'];
			}
		} else {
			// we're getting results from a saved config file
			$services = array();
			foreach($options as $option => $specification) {
				if(!is_numeric($option)) {
					// we hit an option that is not a graph, continue
					continue;
				}
				foreach(explode(',', $specification['host_name']) as $host) {
					foreach(explode(',', $specification['service_desc']) as $service) {
						if ($service == '_HOST_')
							$services[$host] = $host;
						else
							$services["$host;$service"] = "$host;$service";
					}
				}
			}
			$this->obj['services'] = array_keys($services);
		}
		return $this->obj;
	}

	/**
	 * @return boolean
	 */
	function delete_object() {
		if($this->id == 'new') {
			$this->delete_message = "Couldn't find page object";
			return false;
		}
		$file = self::get_filename($this->id);
		if(!file_exists($file)) {
			$this->delete_message = "The file '".$file."' does not exist";
			return false;
		}
		if(!is_writable($file)) {
			$this->delete_message = "I don't have sufficient rights for removing the file ".$file;
			return false;
		}
		if(!unlink($file)) {
			$this->delete_message = "Tried to remove '$file' and failed even though the permissions are correct";
			return false;
		}

		return true;
	}

	/**
	 * @return boolean
	 */
	function banish_object() {
		if (!$this->delete_object())
			return false;
		$this->delete_message = array('obj' => array());
		return true;
	}

	function get_cmd_bar_links() {
		$links = parent::get_cmd_bar_links();
		unset($links['propagate']); // It uses db object lookup, and is of questionable value
		return $links;
	}

	public static function get_objects_for_user($write = false) {
		$ret = array();
		$page_configs = opendir(self::$page_dir);
		while ($page_fname = readdir($page_configs)) {
			$page = fopen(self::$page_dir.$page_fname, 'r');
			$head = fread($page, 7);
			if ($head === '#NACOMA') {
				$name = basename($page_fname, '.cfg');
				$ret[$name] = $name;
			}
			fclose($page);
		}
		closedir($page_configs);
		return $ret;
	}

	/**
	 * @return string
	 */
	function get_object_name() {
		return isset($this->obj['page_name']) ? $this->obj['page_name'] : 'New Graph Collection';
	}

	/**
	 * Does not write to db, write to file
	 *
	 * @param $user = true boolean
	 * @return string full path of newly created page
	 */
	function save_object($user=true) {
		if(!$this->obj['page_name']) {
			return $this->validation_error('invalid object', "Couldn't find object");
		}

		if (self::get_filename($this->obj['page_name']) !== self::get_filename($this->id) || $this->id == 'new') {
			if (file_exists(self::get_filename($this->obj['page_name']))) {
				return $this->validation_error('page_name', "There is already a graph collection with this name, refusing to overwrite");
			}
		}

		if ($this->obj['page_name'] !== $this->id && $this->id && $this->id != 'new') {
			if (!$this->delete_object()) {
				return $this->validation_error('permission', "Couldn't delete previous file name {$this->id}, refusing to write new copy with other name: " . $this->delete_message);
			}
		}

		$obj = $this->denormalize();

		$use_regex = (int)(isset($obj['use_regex']) && $obj['use_regex']);
		$content = "#NACOMA
define page {
	use_regex ".$use_regex."
	page_name ".$obj['page_name']."
}";

		if($use_regex) {
			$content .= "
define graph {
	host_name ".$obj['host_name']."
	service_desc ".$obj['service_desc']."
}";
		} else {
			// we've got values from selectboxes, N:N between services and hosts
			$graphs = array();
			foreach($obj['services'] as $service) {
				if (!strpos($service, ';')) {
					$host = $service;
					$svc = '_HOST_';
				} else {
					list($host, $svc) = explode(';', $service);
				}
				$graphs[] = "
define graph {
	host_name ".$host."
	service_desc $svc
}";
			}
			$content .= implode("\n", $graphs);
		}
		$filename = self::get_filename($obj['page_name']);
		if(!file_put_contents($filename, $content, LOCK_EX)) {
			return $this->validation_error('permission', "Could not write to $filename, please check the permissions");
		}

		$this->id = $this->obj['page_name'];

		return true;
	}
}
