<?php

	/**
	 *  Contains all information required for plugin_search, may
	 *	also contain the information now used by metadata.php?
	 */
	class Plugin_Model {

		/**
		 * The glob used to find all plugins
		 */
		public static $PLUGIN_DIR_GLOB			= "/opt/plugins/*";

		/**
		 * The glob used to find all metadata files
		 */
		public static $METADATA_DIR_GLOB		= "/opt/plugins/metadata/*.metadata";

		/**
		 * The glob used to find all custom plugins
		 */
		public static $PLUGIN_CUSTOM_DIR_GLOB	= "/opt/plugins/custom/*";

		/**
		 * The URL to edit a command (sprintf the command name)
		 */
		public static $COMMAND_URL = "edit.php?obj_type=command&master_obj_id=&regexp1=Filter+by+regular+expression&obj_id=%s&action=Go&action_dummy=Go";

		/**
		 * Used as a container for all found metadata paths
		 */
		public static $METADATA	= false;

		/**
		 * Used as a container for all found plugin paths
		 */
		public static $PLUGINS	= false;

		/**
		 * Used as a container for all found custom plugin paths
		 */
		public static $CUSTOMS	= false;

		/**
		 * Used as a container for all found DB stored commands
		 */
		private static $dbcommands		= array();

		/**
		 *  Construct either a blank plugin or takes the path to
		 *  a metadata file and parses it to populate the plugin
		 *  information
		 */
		public function __construct ($metafile = false) {

			$this->support = "Unsupported";
			$this->supportlevel = 0;

			$this->name		= "unnamed";
			$this->commands	= array();

			$this->info			= "";
			$this->why			= "";
			$this->how			= "";
			$this->platforms	= "";
			$this->categories	= "";
			$this->tags			= array();

			$this->relevance	= 0;
			$this->custom		= false;

			if ( $metafile != false )
				$this->parse( $metafile );

		}

		/**
		 * Will populate static members of the Plugin class
		 * with all paths to plugins, metadata and custom plugins
		 */
		public static function load_plugin_data () {

			self::$METADATA	= glob(self::$METADATA_DIR_GLOB);

			self::$CUSTOMS = array_diff(
				glob(self::$PLUGIN_CUSTOM_DIR_GLOB),
				glob(self::$PLUGIN_CUSTOM_DIR_GLOB, GLOB_ONLYDIR)
			);

			self::$PLUGINS = array_diff(
				glob(self::$PLUGIN_DIR_GLOB),
				glob(self::$PLUGIN_DIR_GLOB, GLOB_ONLYDIR)
			);

		}

		/**
		 * Loads configured commands from the database, not used
		 * in plugin search but can be used instead of metadata.php
		 * to diff configured commands vs. metadata commands
		 */
		public static function load_config_commands () {

			$result = nacoma_db::exec_query( 'SELECT command_name, command_line FROM command' );

			while( $row = $result->fetch( PDO::FETCH_ASSOC ) ) {
				self::$dbcommands[ $row['command_name'] ] = $row['command_line'];
			}

		}

		/**
		 * Parses tags
		 */
		private function handle_tags () {

			if ( is_string( $this->tags ) ) {

				$this->tags = explode( "\n", $this->tags );
				$formatted = array();

				foreach ( $this->tags as $tag ) {

					$tag = explode( "=", $tag );

					if ( count( $tag ) == 2 ) {

						$check = trim( $tag[ 0 ] );
						$t = trim( $tag[ 1 ] );

						$this->commands[ $check ][ "tag" ] = $t;

					} else {

						$tag = implode( "", $tag );
						$tag = explode( ",", $tag );
						$formatted = $tag;

					}

				}

				$this->tags = $formatted;

			}

		}

		/**
		 * Parses commands
		 */
		private function handle_commands () {

			$formatted = array();

			foreach ( $this->commands as $command ) {

				$command = explode( "=", $command );

				$check = array_shift( $command );
				$line = implode( "=", $command );

				$formatted[ $check ] = array(
					"line" => trim( $line ),
					"state" => "unchanged"
				);

			}

			$this->commands = $formatted;

			foreach ( $this->commands as $check => $command ) {

				if ( !isset( self::$dbcommands[ $check ] ) ) {
					$this->commands[ $check ][ "state" ] = "new";
				} elseif ( $command[ "line" ] != self::$dbcommands[ $check ] ) {
					$this->commands[ $check ][ "state" ] = "mod";
				}

			}

		}

		/**
		 * Parses the entirety of a metadata file
		 */
		private function parse ( $metafile ) {

			$handle = fopen( $metafile, 'r' );
			$plugin = basename( preg_replace('/.metadata/',"",$metafile) );
			$result_array = array();
			$section = "";

			$this->name = $plugin;

			if ( $handle ) {

				while (!feof($handle)) {

					$line = fgets($handle);

					/* Strip comments (lines starting with # or ; and blank lines*/
					if (preg_match("/^[#;]/", $line) || trim($line) === "")
						continue;

					/* Look for section header, enclosed in [] */
					if (preg_match("/^[\[][a-zA-Z0-9\-]*[\]][ \t]*$/", $line)){
						$section = trim( $line, " \t[]\n" );
						continue;
					}

					if ( $section == "category" || $section == "categories" )
						$section = "categories";
					elseif ( $section == "platforms" || $section == "platform" )
						$section = "platforms";
					elseif ( $section == "info" || $section == "information" )
						$section = "info";

					if ( !isset($result_array[$section]) )
						$result_array[ $section ] = array();
					$result_array[$section][] = trim( $line );

				}

				fclose($handle);

			}

			foreach ( $result_array as $section => $content ) {
				if ( $section != "commands" )
					$this->$section = implode( "\n", $content );
				else
					$this->commands = $content;
			}

			$this->categories = preg_split( "/\,/", $this->categories );

			if ( $this->support == 4 ) {
				$this->support = "Full";
				$this->supportlevel = 4;
			} elseif ( $this->support == 5 ) {
				$this->support = "Bug Support";
				$this->supportlevel = 5;
			} elseif ( $this->support == 6 ) {
				$this->support = "Best Effort";
				$this->supportlevel = 6;
			}  elseif ( $this->support == 8 ) {
				$this->support = "Deprecated";
				$this->supportlevel = 8;
			} else {
				$this->support = "Unsupported";
				$this->supportlevel = 0;
			}

			$this->handle_commands();
			$this->handle_tags();

		}

	}
